package com.fjhx.flow.service.flow.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.flow.entity.flow.dto.FlowDefinitionDto;
import com.fjhx.flow.entity.flow.dto.FlowDefinitionSelectDto;
import com.fjhx.flow.entity.flow.po.FlowDefinition;
import com.fjhx.flow.entity.flow.po.FlowDefinitionNode;
import com.fjhx.flow.entity.flow.po.FlowInfo;
import com.fjhx.flow.entity.flow.vo.FlowDefinitionPageVo;
import com.fjhx.flow.entity.flow.vo.FlowDefinitionVo;
import com.fjhx.flow.enums.HandleTypeEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.fjhx.flow.mapper.flow.FlowDefinitionMapper;
import com.fjhx.flow.service.flow.FlowDefinitionNodeService;
import com.fjhx.flow.service.flow.FlowDefinitionService;
import com.fjhx.flow.service.flow.FlowInfoService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;


/**
 * <p>
 * 流程定义 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-15
 */
@Service
public class FlowDefinitionServiceImpl extends ServiceImpl<FlowDefinitionMapper, FlowDefinition> implements FlowDefinitionService {

    @Autowired
    private FlowInfoService flowInfoService;

    @Autowired
    private FlowDefinitionNodeService flowDefinitionNodeService;

    @Override
    public Page<FlowDefinitionPageVo> getPage(FlowDefinitionSelectDto dto) {

        List<Long> flowInfoIdList = flowInfoService.listObject(BaseIdPo::getId, q -> q.eq(FlowInfo::getStatus, StatusConstant.ENABLE));
        if (flowInfoIdList.size() == 0) {
            return new Page<>();
        }

        IWrapper<Object> wrapper = IWrapper.getWrapper()
                .keyword(dto, new SqlField("fi", FlowInfo::getFlowKey), new SqlField("fi", FlowInfo::getFlowName))
                .eq("fd", FlowDefinition::getCurrentVersion, StatusConstant.YES)
                .eq("fd", FlowDefinition::getTenantId, dto.getTenantId())
                .in("fd", FlowDefinition::getFlowInfoId, flowInfoIdList);
        return baseMapper.getPage(dto.getPage(), wrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FlowDefinition flowDefinition) {
        Long flowInfoId = flowDefinition.getFlowInfoId();
        String tenantId = flowDefinition.getTenantId();

        long count = count(q -> q.eq(FlowDefinition::getFlowInfoId, flowInfoId).eq(FlowDefinition::getTenantId, tenantId));

        if (count > 0) {
            throw new ServiceException("流程已添加");
        }

        FlowInfo flowInfo = flowInfoService.getById(flowInfoId);
        if (flowInfo == null) {
            throw new ServiceException("没有找到流程定义");
        }

        flowDefinition.setId(IdWorker.getId());
        flowDefinition.setFlowKey(flowInfo.getFlowKey());
        flowDefinition.setVersionNumber(1);
        flowDefinition.setCurrentVersion(StatusConstant.YES);
        this.save(flowDefinition);

        // 添加默认流程节点
        FlowDefinitionNode startNode = new FlowDefinitionNode();
        startNode.setId(IdWorker.getId());
        startNode.setFlowDefinitionId(flowDefinition.getId());
        startNode.setNodeName("发起申请");
        startNode.setNodeType(NodeTypeEnum.START.getKey());
        startNode.setNodeButtonSet(HandleTypeEnum.SKIP_TO_NEXT.getValue());
        startNode.setParentId(0L);

        FlowDefinitionNode endNode = new FlowDefinitionNode();
        endNode.setId(IdWorker.getId());
        endNode.setFlowDefinitionId(flowDefinition.getId());
        endNode.setNodeName("结束");
        endNode.setNodeType(NodeTypeEnum.END.getKey());
        endNode.setParentId(startNode.getId());

        flowDefinitionNodeService.saveBatch(Arrays.asList(startNode, endNode));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addVersion(FlowDefinitionDto dto) {
        Long flowInfoId = dto.getFlowInfoId();
        String tenantId = dto.getTenantId();

        FlowDefinition flowDefinition = getOne(q -> q
                .eq(FlowDefinition::getFlowInfoId, flowInfoId)
                .eq(FlowDefinition::getTenantId, tenantId)
                .orderByDesc(FlowDefinition::getVersionNumber));

        if (flowDefinition == null) {
            throw new ServiceException("未添加流程，无法新增流程版本");
        }

        dto.setId(IdWorker.getId());
        dto.setFlowKey(flowDefinition.getFlowKey());
        dto.setVersionNumber(flowDefinition.getVersionNumber() + 1);
        dto.setCurrentVersion(StatusConstant.NO);
        this.save(dto);

        // 添加流程节点
        List<FlowDefinitionNode> flowDefinitionNodeList = dto.getFlowDefinitionNodeList();

        for (FlowDefinitionNode flowDefinitionNode : flowDefinitionNodeList) {
            flowDefinitionNode.setFlowDefinitionId(dto.getId());

            Long oldId = flowDefinitionNode.getId();
            long newId = IdWorker.getId();
            flowDefinitionNode.setId(newId);

            for (FlowDefinitionNode item : flowDefinitionNodeList) {
                if (item.getParentId().equals(oldId)) {
                    item.setParentId(newId);
                }
            }
        }

        flowDefinitionNodeService.saveBatch(flowDefinitionNodeList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateVersion(Long flowDefinitionId) {
        if (ObjectUtil.isEmpty(flowDefinitionId)) {
            throw new ServiceException("租户流程id不能为空");
        }

        FlowDefinition flowDefinition = getById(flowDefinitionId);
        if (StatusConstant.YES.equals(flowDefinition.getCurrentVersion())) {
            return;
        }

        update(q -> q
                .eq(FlowDefinition::getFlowInfoId, flowDefinition.getFlowInfoId())
                .eq(FlowDefinition::getTenantId, flowDefinition.getTenantId())
                .ne(BaseIdPo::getId, flowDefinition.getId())
                .set(FlowDefinition::getCurrentVersion, StatusConstant.NO)
        );

        flowDefinition.setCurrentVersion(StatusConstant.YES);
        updateById(flowDefinition);
    }

    @Override
    public List<FlowDefinition> getVersionList(FlowDefinition flowDefinition) {
        String flowKey = flowDefinition.getFlowKey();

        if (StrUtil.isBlank(flowKey)) {
            throw new ServiceException("流程key不能为空");
        }

        String tenantId = flowDefinition.getTenantId();
        if (StrUtil.isBlank(tenantId)) {
            throw new ServiceException("租户id不能为空");
        }

        return list(q -> q.eq(FlowDefinition::getFlowKey, flowKey).eq(FlowDefinition::getTenantId, tenantId));
    }

    @Override
    public FlowDefinitionVo getDetails(BaseSelectDto dto) {
        Long id = dto.getId();
        if (id == null) {
            throw new ServiceException("流程定义id不能为空");
        }
        FlowDefinition flowDefinition = getById(id);
        if (flowDefinition == null) {
            throw new ServiceException("没有找到流程定义");
        }

        FlowDefinitionVo flowDefinitionVo = BeanUtil.toBean(flowDefinition, FlowDefinitionVo.class);

        List<FlowDefinitionNode> flowDefinitionNodeList = flowDefinitionNodeService.list(q -> q
                .eq(FlowDefinitionNode::getFlowDefinitionId, id));

        flowDefinitionVo.setFlowDefinitionNodeList(flowDefinitionNodeList);

        return flowDefinitionVo;
    }

}
