package com.fjhx.socket.core;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fjhx.socket.core.event.WebSocketOnMessageEvent;
import com.fjhx.socket.core.event.WebSocketOnOpenEvent;
import com.fjhx.socket.core.event.WebSocketSendFailEvent;
import com.fjhx.socket.core.event.WebSocketSendSuccessEvent;
import com.fjhx.socket.entity.push.po.PushInfo;
import com.fjhx.socket.service.push.PushInfoService;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.framework.web.service.TokenService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint("/webStock/{token}")
@Component
@Slf4j
@Getter
public class WebSocketServer {

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象
     */
    protected static final Map<Long, List<WebSocketServer>> webSocketMap = new ConcurrentHashMap<>();

    private static final ApplicationContext applicationContext = SpringUtil.getApplicationContext();
    private static final TokenService tokenService = SpringUtil.getBean(TokenService.class);
    private static final PushInfoService pushInfoService = SpringUtil.getBean(PushInfoService.class);

    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * sessionId
     */
    private Long sessionId;

    /**
     * 租户id
     */
    private String tenantId;

    /**
     * userId
     */
    private Long userId;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(@RequestBody Session session, @PathParam("token") String token) {

        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);

        // 解析token
        LoginUser loginUser = tokenService.getLoginUser(token);

        // token错误
        if (loginUser == null) {
            try {
                PushParam pushParam = new PushParam();
                pushParam.setTitle("token错误");
                pushParam.setCreateTime(new Date());
                this.sendMessage(PushTypeEnum.SYSTEM, pushParam);
                this.session.close();
            } catch (Exception e) {
                log.error("webSocket连接失败", e);
            }
            return;
        }

        SysUser user = loginUser.getUser();
        this.session = session;
        this.sessionId = IdWorker.getId();
        this.userId = user.getUserId();
        this.tenantId = user.getTenantId();

        synchronized (WebSocketServer.class) {
            List<WebSocketServer> webSocketServerList = webSocketMap.computeIfAbsent(userId, k -> new ArrayList<>());
            webSocketServerList.add(this);
        }

        // 发布建立连接连接事件
        applicationContext.publishEvent(new WebSocketOnOpenEvent(this));

        DynamicDataSourceContextHolder.poll();
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        synchronized (WebSocketServer.class) {
            List<WebSocketServer> webSocketServerList = webSocketMap.get(userId);
            webSocketServerList.remove(this);

            if (webSocketServerList.size() == 0) {
                webSocketMap.remove(userId);
            }
        }
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message) {
        // 推送消息事件
        applicationContext.publishEvent(new WebSocketOnMessageEvent(this, message));
    }

    /**
     * @param error 异常
     */
    @OnError
    public void onError(Throwable error) {
        log.error("webSocket连接异常", error);
    }

    /**
     * 实现服务器主动推送
     */
    protected void sendMessage(PushTypeEnum pushTypeEnum, PushParam pushParam) {

        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);

        JSONObject msg = new JSONObject();
        msg.put("type", pushTypeEnum.getType());

        switch (pushTypeEnum) {
            case SYSTEM:
                msg.put("content", pushParam.getTitle());
                break;
            case ANNOUNCEMENT:
                List<PushInfo> list = pushInfoService.list(q -> q
                        .eq(PushInfo::getType, PushTypeEnum.ANNOUNCEMENT.getType())
                        .eq(PushInfo::getPushUserId, pushParam.getUserId())
                        .eq(PushInfo::getPushRead, StatusConstant.NO)
                        .and(s -> s.isNull(PushInfo::getFailureTime).or().ge(PushInfo::getFailureTime, new Date()))
                );
                msg.put("list", list);
                break;
            case MESSAGE:
                Long count = pushInfoService.count(q -> q
                        .eq(PushInfo::getType, PushTypeEnum.MESSAGE.getType())
                        .eq(PushInfo::getPushUserId, pushParam.getUserId())
                        .eq(PushInfo::getPushRead, StatusConstant.NO)
                        .and(s -> s.isNull(PushInfo::getFailureTime).or().ge(PushInfo::getFailureTime, new Date()))
                );
                msg.put("count", count);
                break;
            case PUSH_NOTIFICATION:
                msg.put("title", pushParam.getTitle());
                msg.put("businessType", pushParam.getBusinessType());
                msg.put("businessData", pushParam.getBusinessData());
                break;
        }

        try {
            this.session.getBasicRemote().sendText(msg.toJSONString(JSONWriter.Feature.WriteLongAsString));
            applicationContext.publishEvent(new WebSocketSendSuccessEvent(pushTypeEnum, pushParam));
        } catch (IOException e) {
            applicationContext.publishEvent(new WebSocketSendFailEvent(pushTypeEnum, pushParam));
        }

        DynamicDataSourceContextHolder.poll();
    }

}
