package com.ruoyi.framework.mybatis.interceptor;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.druid.DbType;
import com.alibaba.druid.sql.SQLUtils;
import com.baomidou.mybatisplus.core.toolkit.PluginUtils;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.mapping.MappedStatement;
import org.apache.ibatis.mapping.ParameterMode;
import org.apache.ibatis.plugin.Interceptor;
import org.apache.ibatis.plugin.Intercepts;
import org.apache.ibatis.plugin.Invocation;
import org.apache.ibatis.plugin.Signature;
import org.apache.ibatis.reflection.MetaObject;
import org.apache.ibatis.reflection.SystemMetaObject;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.session.ResultHandler;
import org.apache.ibatis.type.TypeHandlerRegistry;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

import java.sql.Statement;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 打印执行的sql日志
 */
@Intercepts({
        @Signature(type = StatementHandler.class, method = "query", args = {Statement.class, ResultHandler.class}),
        @Signature(type = StatementHandler.class, method = "update", args = {Statement.class}),
        @Signature(type = StatementHandler.class, method = "batch", args = {Statement.class})
})
@Profile("dev")
@Slf4j
@Component
public class SqlLogInterceptor implements Interceptor {

    private final SQLUtils.FormatOption FORMAT_OPTION = new SQLUtils.FormatOption(false, false);

    @Override
    public Object intercept(Invocation invocation) throws Throwable {

        long start = System.currentTimeMillis();
        Object result = invocation.proceed();
        long end = System.currentTimeMillis();

        StatementHandler statementHandler = PluginUtils.realTarget(invocation.getTarget());
        MetaObject metaObject = SystemMetaObject.forObject(statementHandler);
        BoundSql boundSql = (BoundSql) metaObject.getValue("delegate.boundSql");

        Object parameterObject = boundSql.getParameterObject();
        if (parameterObject == null) {
            log.error("parameterObject is null");
            return result;
        }

        MappedStatement mappedStatement = (MappedStatement) metaObject.getValue("delegate.mappedStatement");
        Configuration configuration = mappedStatement.getConfiguration();
        TypeHandlerRegistry typeHandlerRegistry = configuration.getTypeHandlerRegistry();
        MetaObject newMetaObject = configuration.newMetaObject(parameterObject);

        List<Object> parameters = boundSql.getParameterMappings().stream()
                .filter(item -> ObjectUtil.notEqual(item.getMode(), ParameterMode.OUT))
                .map(item -> {
                    String propertyName = item.getProperty();
                    if (typeHandlerRegistry.hasTypeHandler(parameterObject.getClass())) {
                        return getParameterValue(parameterObject);
                    } else if (newMetaObject.hasGetter(propertyName)) {
                        return getParameterValue(newMetaObject.getValue(propertyName));
                    } else if (boundSql.hasAdditionalParameter(propertyName)) {
                        return getParameterValue(boundSql.getAdditionalParameter(propertyName));
                    }
                    return StringPool.EMPTY;
                }).collect(Collectors.toList());

        String sql = SQLUtils.format(boundSql.getSql(), DbType.mysql, parameters, FORMAT_OPTION);

        // 打印 sql
        log.info("\n====================  Sql Start  ===================="
                        + "\n mapper       : {}"
                        + "\n execute sql  : {}"
                        + "\n execute time : {}"
                        + "\n====================  Sql  End   ====================\n",
                mappedStatement.getId(), sql, DateUtil.formatBetween(end - start));

        return result;
    }

    /**
     * 获取参数
     *
     * @param obj Object类型参数
     * @return 转换之后的参数
     */
    private Object getParameterValue(Object obj) {
        if (obj == null) {
            return StringPool.EMPTY;
        }
        if (obj instanceof Date) {
            return DateUtil.formatDateTime((Date) obj);
        }
        return obj;
    }

}
